@@ -14,9 +14,25 @@
 """
 $Id:$
 """
-__docformat__ = "reStructuredText"
+from __future__ import division
+from __future__ import absolute_import
+from future import standard_library
+standard_library.install_aliases()
 
-import thread
+from builtins import str
+from past.utils import old_div
+from future.utils import PY2
+from future.types import newstr
+try:
+    unicode
+except NameError:
+    # Python 3
+    unicode = str
+
+import six
+from six import string_types
+
+import _thread
 import datetime
 
 import pymongo
@@ -87,7 +103,6 @@ class MongoParentAware(object):
     _pid = None
     _m_parent = None
 
-    @apply
     def __parent__():
         def fget(self):
             return self._m_parent
@@ -97,10 +112,12 @@ class MongoParentAware(object):
             if self._pid is None:
                 try:
                     self._pid = __parent__._mpid
-                except AttributeError, e:
+                except AttributeError as e:
                     pass
         return property(fget, fset)
 
+    __parent__ = __parent__()
+
 
 class SetupConvert(zope.location.location.Location):
     """Mixin class for setup, update and dump item and subitems.
@@ -183,7 +200,7 @@ class SetupConvert(zope.location.location.Location):
 
         # setup key/values
         skipNames = self._skipNames + self.skipNames
-        for k, v in data.items():
+        for k, v in list(data.items()):
             # skip names if given in data dict
             if k not in skipNames:
                 # dump old data
@@ -197,15 +214,15 @@ class SetupConvert(zope.location.location.Location):
                 # set value
                 try:
                     setattr(self, k, v)
-                except zope.interface.Invalid, e:
+                except zope.interface.Invalid as e:
                     pass
-                except (AttributeError, ValueError, zope.interface.Invalid), e:
+                except (AttributeError, ValueError) as e:
                     raise AttributeError(
                         "Can't set value %s %s for attr %s: %r on %r" % (
                             v, type(v), k, str(e), self))
 
         # this allows to ensure default values or even converter instances
-        for k, v in self.defaults.items():
+        for k, v in list(self.defaults.items()):
             if getattr(self, k, None) is None:
                 # convert value or return an initial empty converter instance
                 v = self.convert(k, v)
@@ -241,7 +258,8 @@ class Dump(object):
             dumpNames = self._dumpNames + self.dumpNames
         # allways preserve order in mongodb
         data = bson.son.SON()
-        for k in sorted(dumpNames, key=str.lower):
+        to_lower = lambda x: x.lower() if isinstance(x, str) else x.decode('utf-8').lower()
+        for k in sorted(dumpNames, key=to_lower):
             # get value from __dict__, this will ensure that we catch values
             # converted by field properties e.g. MongoDateProperty
             v = self.__dict__.get(k, NO_VALUE_MARKER)
@@ -269,6 +287,7 @@ class SetupConvertDump(SetupConvert, Dump):
     """Mixin class for update items without transaction support"""
 
 
+@zope.interface.implementer(interfaces.IMongoUpdateMixin)
 class MongoUpdateMixin(object):
     """Optional update method mixin class.
 
@@ -278,7 +297,6 @@ class MongoUpdateMixin(object):
     if the transaction runs into a retry loop.
     """
 
-    zope.interface.implements(interfaces.IMongoUpdateMixin)
 
     def update(self, data, raiseErrors=False, **kwargs):
         """Update an object with given data without using the built in
@@ -303,7 +321,7 @@ class MongoUpdateMixin(object):
                 dump = self.__parent__.doUpdateDump(self)
                 self.__parent__.doUpdate(collection, dump, upsert=False)
         except (TypeError, pymongo.errors.ConnectionFailure,
-            pymongo.errors.AutoReconnect, pymongo.errors.InvalidName), e:
+            pymongo.errors.AutoReconnect, pymongo.errors.InvalidName) as e:
             # revert data, the transaction will try to store them later
             self.setup(orgData)
             if raiseErrors:
@@ -377,7 +395,7 @@ class MongoItemBase(MongoParentAware, SetupConvertDump):
         # set given or new created datetime
         created = data.pop('created', _marker)
         if created is _marker:
-            created = datetime.datetime.now(UTC)
+            created = m01.mongo.now(UTC)
         self.__dict__['created'] = created
 
         # setup object with given (non IMongoItem) key/value data
@@ -458,7 +476,6 @@ class MongoSubObjectBase(MongoParentAware, SetupConvertDump):
         # access will end in a write access.
         self._m_changed = None
 
-    @apply
     def _m_changed():
         def get(self):
             return self.__m_changed
@@ -468,7 +485,10 @@ class MongoSubObjectBase(MongoParentAware, SetupConvertDump):
                 self.__parent__._m_changed = True
         return property(get, set)
 
+    _m_changed = _m_changed()
 
+
+@zope.interface.implementer(interfaces.IMongoTransactionAware)
 class MongoMappingBase(MongoItemAware, zope.location.location.Location):
     """Mongo mapping base class with thread save transaction and caching
     support.
@@ -486,7 +506,6 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
 
     """
 
-    zope.interface.implements(interfaces.IMongoTransactionAware)
 
     @property
     def collection(self):
@@ -508,7 +527,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
         instance.
         """
         return '%s.%i.%i' % (self.collection.full_name, id(self),
-            thread.get_ident())
+            _thread.get_ident())
 
     def load(self, data):
         """Load data into a IMongoItem
@@ -639,7 +658,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
             spec = bson.son.SON()
         if not skipFilter:
             spec = self.doCountFilter(spec)
-        return collection.count(spec)
+        return collection.count_documents(spec)
 
     def doFindOne(self, collection, spec=None, fields=None, skipFilter=False):
         if spec is None:
@@ -762,7 +781,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
                     continue
             yield __name__, obj
         # also return keys/items not stored in MongoDB yet
-        for k, v in self._cache_added.items():
+        for k, v in list(self._cache_added.items()):
             yield k, v
 
     def keys(self):
@@ -774,7 +793,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
             else:
                 yield __name__
         # also return keys not stored in MongoDB yet
-        for k, v in self._cache_added.items():
+        for k, v in list(self._cache_added.items()):
             yield k
 
     def values(self):
@@ -794,13 +813,13 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
                     continue
             yield obj
         # also return items not stored in MongoDB yet
-        for k, v in self._cache_added.items():
+        for k, v in list(self._cache_added.items()):
             yield v
 
     def __iter__(self):
         """Return an iterator for the keys of the mapping object.
         """
-        return iter(self.keys())
+        return iter(list(self.keys()))
 
     def __getitem__(self, key):
         """get item"""
@@ -809,7 +828,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
     def get(self, key, default=None):
         try:
             return self[key]
-        except KeyError, e:
+        except KeyError as e:
             pass
         return default
 
@@ -824,6 +843,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
             # remove new items
             del self._cache_added[key]
         else:
+            # import pdb; pdb.set_trace()
             # or add to remove cache but only if not yet added
             self._cache_removed[key] = obj
         # also remove from loaded items
@@ -876,9 +896,9 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
         cursor = self.collection.find(query, fields)
 
         # get overall total based on query
-        total = cursor.count()
+        total = self.collection.count_documents({})
         # calculate pages
-        pages = total/size
+        pages = old_div(total,size)
         if pages == 0 or total % size:
             pages += 1
         # as next we approve our page position
@@ -894,7 +914,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
         # sort result
         if sortName is not None:
             if sortOrder is None:
-                if isinstance(sortName, basestring):
+                if isinstance(sortName, six.string_types):
                     sortName = [(sortName, pymongo.ASCENDING)]
                 elif not isinstance(sortName, list):
                     raise TypeError("if no sortOrder is specified, "
@@ -936,8 +956,8 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
         self.resetTransactionCache()
         ci = self._t_cache_insert
         cu = self._t_cache_update
-        dt = datetime.datetime.now(UTC)
-        for obj in self._cache_loaded.values():
+        dt = m01.mongo.now(UTC)
+        for obj in list(self._cache_loaded.values()):
             if obj._m_changed:
                 # set modified
                 obj.modified = dt
@@ -950,7 +970,7 @@ class MongoMappingBase(MongoItemAware, zope.location.location.Location):
 
         # add removed objects specification to transaction remove cache
         cr = self._t_cache_remove
-        for obj in self._cache_removed.values():
+        for obj in list(self._cache_removed.values()):
             cr.append(self.doRemoveDump(obj))
 
         # reset load cache which will support a clean retry
@@ -1005,11 +1025,16 @@ class MongoStorageBase(MongoMappingBase):
             return True
         else:
             try:
+                if PY2 and isinstance(key, newstr):
+                    key_str = "{}".format(key)  # ObjectId requires a simple str
+                else:
+                    key_str = str(key)  # Standard conversion in Python 3
+
                 data = self.doFindOne(self.collection,
-                    {'_id': bson.objectid.ObjectId(str(key))})
+                    {'_id': bson.objectid.ObjectId(key_str)})
                 if data is not None:
                     return True
-            except(pymongo.errors.InvalidId, TypeError), e:
+            except(pymongo.errors.InvalidId, TypeError) as e:
                 pass
 
         return False
@@ -1042,11 +1067,16 @@ class MongoStorageBase(MongoMappingBase):
         obj = self._cache_loaded.get(key)
         if obj is None:
             try:
+                if PY2 and isinstance(key, newstr):
+                    key_str = "{}".format(key)  # ObjectId requires a simple str
+                else:
+                    key_str = str(key)  # Standard conversion in Python 3
+
                 data = self.doFindOne(self.collection,
-                    {'_id': bson.objectid.ObjectId(str(key))})
+                    {'_id': bson.objectid.ObjectId(key_str)})
                 if data is None:
                     raise KeyError(key)
-            except(pymongo.errors.InvalidId, TypeError), e:
+            except(pymongo.errors.InvalidId, TypeError) as e:
                 raise KeyError(key)
             obj = self.doLoad(data)
             # join transaction handling
@@ -1070,7 +1100,7 @@ class MongoContainerBase(MongoMappingBase):
                 data = self.doFindOne(self.collection, {'__name__': key})
                 if data is not None:
                     return True
-            except(pymongo.errors.InvalidId, TypeError), e:
+            except(pymongo.errors.InvalidId, TypeError) as e:
                 pass
 
         return False
@@ -1108,7 +1138,7 @@ class MongoContainerBase(MongoMappingBase):
                 data = self.doFindOne(self.collection, {'__name__': key})
                 if data is None:
                     raise KeyError(key)
-            except(pymongo.errors.InvalidId, TypeError), e:
+            except(pymongo.errors.InvalidId, TypeError) as e:
                 raise KeyError(key)
             obj = self.doLoad(data)
             # join transaction handling
