@@ -14,6 +14,15 @@
 """
 $Id:$
 """
+from __future__ import absolute_import
+from builtins import str
+from builtins import object
+try:
+    unicode
+except NameError:
+    # Python 3
+    unicode = str
+
 __docformat__ = "reStructuredText"
 
 import datetime
@@ -120,7 +129,7 @@ class MongoFieldProperty(object):
         # now validate
         field = self.__field.bind(inst)
         field.validate(value)
-        if field.readonly and inst.__dict__.has_key(self.__name):
+        if field.readonly and self.__name in inst.__dict__:
             raise ValueError(self.__name, 'field is readonly')
         # apply changes
         self.applyChanges(inst, value)
@@ -183,7 +192,7 @@ class MongoBinaryProperty(object):
         # validate
         field = self.__field.bind(inst)
         field.validate(value)
-        if field.readonly and inst.__dict__.has_key(self.__name):
+        if field.readonly and self.__name in inst.__dict__:
             raise ValueError(self.__name, 'field is readonly')
         # apply changes
         self.applyChanges(inst, value)
@@ -251,7 +260,7 @@ class MongoDateProperty(object):
             value = datetime.date.fromordinal(value)
         field = self.__field.bind(inst)
         field.validate(value)
-        if field.readonly and inst.__dict__.has_key(self.__name):
+        if field.readonly and self.__name in inst.__dict__:
             raise ValueError(self.__name, 'field is readonly')
         if value is not None:
             value = value.toordinal()
@@ -318,7 +327,7 @@ class MongoGeoLocationProperty(object):
         # now validate
         field = self.__field.bind(inst)
         field.validate(value)
-        if field.readonly and inst.__dict__.has_key(self.__name):
+        if field.readonly and self.__name in inst.__dict__:
             raise ValueError(self.__name, 'field is readonly')
         # since z3c.form can't simply check object items, we will only save a
         # geo location if the lat, lon will be different then the current
@@ -400,7 +409,7 @@ class MongoGeoPointProperty(object):
         # now validate
         field = self.__field.bind(inst)
         field.validate(value)
-        if field.readonly and inst.__dict__.has_key(self.__name):
+        if field.readonly and self.__name in inst.__dict__:
             raise ValueError(self.__name, 'field is readonly')
         # since z3c.form can't simply check object items, we will only save a
         # geo location if the lat, lon will be different then the current
@@ -435,8 +444,8 @@ class MongoGeoPointProperty(object):
 class MongoSubObjectProperty(object):
     """Computed attributes based on zope.schema.Object
 
-    The field property will convert the gonve data to an object using the
-    given converter method or class. If None data is ginve for this field,
+    The field property will convert the given data to an object using the
+    given converter method or class. If None data is given for this field,
     the field property does not create and add an object.
 
     The field property get method will create a new object if None is stored
@@ -502,7 +511,7 @@ class MongoSubObjectProperty(object):
         # validate the obj
         field = self.__field.bind(inst)
         field.validate(obj)
-        if field.readonly and inst.__dict__.has_key(self.__name):
+        if field.readonly and self.__name in inst.__dict__:
             raise ValueError(self.__name, 'field is readonly')
         # apply changed obj
         inst.__dict__[self.__name] = obj
