@@ -14,6 +14,7 @@
 """
 $Id:$
 """
+from __future__ import absolute_import
 __docformat__ = "reStructuredText"
 
 import zope.interface
@@ -354,7 +355,7 @@ class IMongoUpdateMixin(zope.interface.Interface):
         transaction handling.
 
         This method is usefull for update key/values given in a data dict
-        which need to get updated during processing. On a sucessfull update,
+        which need to get updated during processing. On a successfull update,
         True is returnd otherwise False.
 
         If raiseErrors is set to True, an error get raised.
