@@ -14,6 +14,9 @@
 """
 $Id:$
 """
+from __future__ import absolute_import
+from __future__ import print_function
+
 __docformat__ = "reStructuredText"
 
 import unittest
@@ -38,22 +41,30 @@ def test_suite():
         'usecase.txt',
         'util.txt',
         'zope.schema.txt',
-        ]
+    ]
     for name in fakeTestNames:
         append(
             doctest.DocFileSuite(name,
+                globs={
+                    'print_function': print_function,
+                    'absolute_import': absolute_import
+                },
                 setUp=m01.mongo.testing.setUpFakeMongo,
                 tearDown=m01.mongo.testing.tearDownFakeMongo,
-                optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
+                optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS,
                 checker=m01.mongo.testing.reNormalizer),
         )
     append(
         doctest.DocFileSuite('testing.txt',
-            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
+            globs={
+                'print_function': print_function,
+                'absolute_import': absolute_import
+            },
+            optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS,
             checker=m01.mongo.testing.reNormalizer),
     )
 
-    # real mongo database tests using m01.stub using level 2 tests (--all)
+    # real mongo database tests using m01.stub using level 2 tests (--all or -a2)
     testNames = [
         '../README.txt',
         'batching.txt',
@@ -65,15 +76,19 @@ def test_suite():
         'storage.txt',
         'usecase.txt',
         'zope.schema.txt',
-        ]
+    ]
     for name in testNames:
         suite = unittest.TestSuite((
             doctest.DocFileSuite(name,
+                globs={
+                    'print_function': print_function,
+                    'absolute_import': absolute_import
+                },
                 setUp=m01.mongo.testing.setUpStubMongo,
                 tearDown=m01.mongo.testing.tearDownStubMongo,
-                optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
+                optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS,
                 checker=m01.mongo.testing.reNormalizer),
-            ))
+        ))
         suite.level = 2
         append(suite)
 
@@ -81,5 +96,5 @@ def test_suite():
     return unittest.TestSuite(suites)
 
 
-if __name__=='__main__':
+if __name__ == '__main__':
     unittest.main(defaultTest='test_suite')
